@@ -1,11 +1,12 @@
 import contextlib
 import itertools
 import sys
+import warnings
 from pathlib import Path
 
 import pytest
 
-from pyjsx.transpiler import transpile
+from pyjsx.transpiler import ParseError, transpile
 
 
 @pytest.mark.parametrize(
@@ -183,9 +184,7 @@ def App():
 )
 def test_multiline(snapshot, request, source):
     snapshot.snapshot_dir = Path(__file__).parent / "data"
-    snapshot.assert_match(
-        transpile(source), f"transpiler-multiline-{request.node.callspec.id}.txt"
-    )
+    snapshot.assert_match(transpile(source), f"transpiler-multiline-{request.node.callspec.id}.txt")
 
 
 @pytest.mark.parametrize(
@@ -211,6 +210,16 @@ def test_jsx_text_escapes(source, expected):
     assert transpile(source) == expected
 
 
+@pytest.mark.parametrize(
+    ("source", "expected"),
+    [
+        ("<li><>foo</></li>", 'jsx("li", {}, [jsx(jsx.Fragment, {}, ["foo"])])'),
+    ],
+)
+def test_child_fragments(source, expected):
+    assert transpile(source) == expected
+
+
 def _get_stdlib_python_modules():
     modules = sys.stdlib_module_names
     for name in modules:
@@ -219,7 +228,8 @@ def _get_stdlib_python_modules():
             continue
 
         module = None
-        with contextlib.suppress(Exception):
+        with contextlib.suppress(Exception), warnings.catch_warnings():
+            warnings.filterwarnings("ignore", category=DeprecationWarning)
             module = __import__(name)
 
         if (file_ := getattr(module, "__file__", None)) is None:
@@ -236,3 +246,44 @@ def _get_stdlib_python_modules():
 def test_roundtrip(module_path):
     source = module_path.read_text("utf-8")
     assert transpile(source) == source
+
+
+def test_mismatched_closing_tags():
+    with pytest.raises(ParseError, match="Expected closing tag </div>, got </span>"):
+        transpile("<div></span>")
+
+
+def test_unclosed_tag():
+    with pytest.raises(ParseError, match="No more tokens"):
+        transpile("<div>")
+
+
+def test_unclosed_tag_with_attributes():
+    with pytest.raises(ParseError, match="No more tokens"):
+        transpile("<div foo='bar'>")
+
+
+def test_unclosed_tag_with_children():
+    with pytest.raises(ParseError, match="No more tokens"):
+        transpile("<div><span />")
+
+
+def test_unclosed_fragment():
+    with pytest.raises(ParseError, match="No more tokens"):
+        transpile("<>")
+
+
+def test_unclosed_fragment_with_children():
+    with pytest.raises(ParseError, match="No more tokens"):
+        transpile("<><span />")
+
+
+@pytest.mark.parametrize(
+    ("source", "expected"),
+    [
+        ("<turbo-frame></turbo-frame>", 'jsx("turbo-frame", {}, [])'),
+        ("<turbo-frame />", 'jsx("turbo-frame", {}, [])'),
+    ],
+)
+def test_custom_elements(source, expected):
+    assert transpile(source) == expected
